/****************************************************************************
**
** ISAC2: Integrated Space and Time Adaptive Chip-Package Thermal Analysis, 
**        version 2.2
**
** Copyright (C) 2018 University of Colorado at Boulder and University 
** of Michigan at Ann Arbor. All rights reserved.
**
** This file is part of ISAC2.
**
** ISAC2 is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, version 2 of the License.
**
** ISAC2 is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with ISAC2 in the file COPYING.  If not, see 
** <http://www.gnu.org/licenses/>.
**
** For feedback, comments, or questions, please contact Zyad Hassan 
** (zyad.hassan@colorado.edu) and Xi Chen (chexi@umich.edu)
**
** Please contact the authors if you are interested in making commercial
** use of ISAC2.  It may be possible to re-release under a different license.
****************************************************************************/

/****************************************************************************
** Time_index.h:
****************************************************************************/

#ifndef TIME_INDEX_H_
#define TIME_INDEX_H_

/*###########################################################################*/

/*	A queue structure, rank all the elements by key vaule all the time, can 
	only do operation on the first, or last element for efficiency 
	consideration */

#include <queue>
#include <utility>

struct TimeIndex {
	TimeIndex(double time_a, int index_a) : time(time_a), index(index_a) {}
		double time;
		int index;
};


struct ti_later :
	public std::binary_function<const TimeIndex &, const TimeIndex &, bool>
{
	bool operator()(const TimeIndex & a, const TimeIndex & b)
		{ return a.time > b.time; }
};


class TIQueue {
public:
	TIQueue();
	void push(const TimeIndex & ti) { impl_.push(ti); }
	void pop() { impl_.pop(); }
	const TimeIndex & smallest() const { return impl_.top(); }

private:
		std::priority_queue<TimeIndex, std::vector<TimeIndex>, ti_later> impl_;
};


#if 0
#include <set>

class TIQueue {
public:
	typedef std::multiset<TimeIndex, ti_later>::iterator iterator;

	iterator push(const TimeIndex & ti) { return impl_.insert(ti); }
	void pop() { impl_.erase(--impl_.end()); }
	const TimeIndex & smallest() const { return *(--impl_.end()); }
	void squash(iterator i) { impl_.erase(i); }

private:
		std::multiset<TimeIndex, ti_later> impl_;
};
#endif

#endif
