/****************************************************************************
**
** ISAC2: Integrated Space and Time Adaptive Chip-Package Thermal Analysis, 
**        version 2.2
**
** Copyright (C) 2018 University of Colorado at Boulder and University 
** of Michigan at Ann Arbor. All rights reserved.
**
** This file is part of ISAC2.
**
** ISAC2 is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, version 2 of the License.
**
** ISAC2 is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with ISAC2 in the file COPYING.  If not, see 
** <http://www.gnu.org/licenses/>.
**
** For feedback, comments, or questions, please contact Zyad Hassan 
** (zyad.hassan@colorado.edu) and Xi Chen (chexi@umich.edu)
**
** Please contact the authors if you are interested in making commercial
** use of ISAC2.  It may be possible to re-release under a different license.
****************************************************************************/

/****************************************************************************
** boundaries.cc: code for the boundaries of the system
****************************************************************************/

#include "header.h"


////---------------Specular boundaries code---------------------////

//*==========================================================================*/
//* updates the energies at specular boundaries
void BTE_structure::specular() {

    int angle_spec; //temporary value

    int N_theta_x_2 = N_theta*2;
    int N_phi_x_4 = N_phi*4;

	if((spec_y_0 == 1 )||(spec_y_end == 1)) {
	    //for all control angles and cells in x and z
    	for(int m=0; m<(N_theta_x_2); m++) {
        	for(int n=0; n<(N_phi_x_4); n++) {
            	//get the correct angle
	            int angle = n + (m*4*N_phi);

    	        for(int i=1; i<cells_x-1; i++) {
        	        for(int k=1; k<cells_z-1; k++) {

            	        //IF Y=0 is a specular boundary:
                	    if(spec_y_0 == 1 ) {

                    	    //if the j component of S is going into the boundary
                        	if(propvec[angle].S_j_ < 0 ) {
                            	//the 'f' of the boundary for this angle equals
	                                //the one just out of the boundary
	                            cell[i][0][k].f_angle[angle] =
    					                        cell[i][1][k].f_angle[angle];

	                       	}
		                        //if the j component of S is not going into the
    		                    //boundary                        
							else if(propvec[angle].S_j_ > 0) {
                	            //find the corresponding angle for specular
                    	            //reflection
                        	    //check if it is in the 1st or 4th octant

                            	//if this angle < # of phi in octant, then it is
                                	//in the 1st octant
	                            if(n<N_phi) {
    	                            //get the opposite (specularly) angle
        	                        angle_spec = 2*N_phi -n -1 ;
            	                }
                	            //else it is in the 4th
                    	        else {
                        	        //get the opposite (specularly) angle
                            	    angle_spec = 6*N_phi - n -1 ;
	                            }

    	                        //put the reflected angle in the correct form
        	                    angle_spec = angle_spec + (m*4*N_phi);
            	                //f(i, bottom boundary,k, this angle) =
                	                //f(i,cell right above this one,k,ref_angle)
                    	        cell[i][0][k].f_angle[angle] =
                                            cell[i][1][k].f_angle[angle_spec];

                       		}
	                    }

    	                //IF Y=cells_y is a specular boundary:
        	            if(spec_y_end == 1 ) {

            	            //if the j component of S is going into the boundary
                	        if(propvec[angle].S_j_ > 0 ) {
                    	        //the 'f' of the boundary for this angle equals
                                //the one just out of the boundary
	                            cell[i][cells_y-1][k].f_angle[angle] =
                                        cell[i][cells_y-2][k].f_angle[angle];
    	                    }
	                        //if the j component of S is not going into the
    	                        //boundary
        	                else if(propvec[angle].S_j_ < 0) {

            	                //find the corresponding angle for specular
                	                //reflection
	                            //check if it is in the 2nd or 3rd octant
	
    	                        //2nd
        	                    if((N_phi<=n)&&(n<2*N_phi)) {
            	                    //get the opposite (specularly) angle
                	                angle_spec = 2*N_phi -n -1 ;
                    	        }

	                            //else it is in the 3rd
    	                        else {
        	                        //get the opposite (specularly) angle
            	                    angle_spec = 6*N_phi - n -1;
                	            }

                    	        //put the reflected angle in the correct form
                        	    angle_spec = angle_spec + (m*4*N_phi);
                            	//f(i, top boundary,k, this angle) =
                                	//f(i,cell right below this one,k,ref_angle)
	                            cell[i][cells_y-1][k].f_angle[angle] =
                                    cell[i][cells_y-2][k].f_angle[angle_spec];

    	                    }

	                    }
    	            }
        	    }
        	}
    	}
	}

    if((spec_x_0 == 1 )||(spec_x_end == 1)) {
	    //for all control angles and cells in y and z
    	for(int m=0; m<(N_theta_x_2); m++) {
        	for(int n=0; n<(N_phi_x_4); n++) {
            	//get the correct angle
	            int angle = n + (m*4*N_phi);

    	        for(int j=1; j<cells_y-1; j++) {
        	        for(int k=1; k<cells_z-1; k++) {

            	        //IF X=0 is a specular boundary:
                	    if(spec_x_0 == 1) {

                    	    //if the i component of S is going into the boundary
                        	if(propvec[angle].S_i_ < 0 ) {
                            	//the 'f' of the boundary for this angle equals
                                	//the one just out of the boundary
	                            cell[0][j][k].f_angle[angle] =
                                                cell[1][j][k].f_angle[angle];
    	                    }
	                        //if the i component of S is not going into the
	                        //boundary                        
							else if(propvec[angle].S_i_ > 0) {
	                            //find the corresponding angle for specular
    	                            //reflection

        	                    //check if it is in the 1st or 2nd octant
            	                //1st
                	            if(n<N_phi) {
                    	            //get the opposite (specularly) angle
                        	        angle_spec = 4*N_phi -n -1;
                            	}
	                            //else it is in the 2nd
    	                        else {
        	                        //get the opposite (specularly) angle
            	                    angle_spec = 4*N_phi - n -1;
                	            }

                    	        //put the reflected angle in the correct form
                        	    angle_spec = angle_spec + (m*4*N_phi);
                            	//f(bottom boundary,j,k, this angle) =
                                	//f(cell right above this one,j,k,ref_angle)
	                            cell[0][j][k].f_angle[angle] =
                                            cell[1][j][k].f_angle[angle_spec];

    	                    }
        	            }
            	        //IF X=cells_x is a specular boundary:
                	    if(spec_x_end == 1 ) {

                    	    //if the i component of S is going into the boundary
                        	if(propvec[angle].S_i_ > 0 ) {
                            	//the 'f' of the boundary for this angle equals
                                	//the one just out of the boundary
	                            cell[cells_x-1][j][k].f_angle[angle] =
                                        cell[cells_x-2][j][k].f_angle[angle];
    	                    }
	                        //if the i component of S is not going into the
    	                        //boundary
        	                else if(propvec[angle].S_i_ < 0) {

            	                //find the corresponding angle for specular
                	                //reflection

                    	        //check if it is in the 3rd or 4th octant
                        	    //3rd
                            	if((2*N_phi<=n)&&(n<3*N_phi)) {
                                	//get the opposite (specularly) angle
	                                angle_spec = 4*N_phi -n -1;
    	                        }
        	                    //else it is in the 4th
            	                else {
                	                //get the opposite (specularly) angle
                    	            angle_spec = 4*N_phi - n -1;
                        	    }

                            	//put the reflected angle in the correct form
	                            angle_spec = angle_spec + (m*4*N_phi);
    	                        //f(top boundary,j,k, this angle) =
        	                        //f(cell right below this one,j,k,ref_angle)
            	                cell[cells_x-1][j][k].f_angle[angle] =
               	                    cell[cells_x-2][j][k].f_angle[angle_spec];
							}
                        }
                    }
                }
            }
        }
    }

    if((spec_z_0 == 1 )||(spec_z_end == 1)) {
	    //for all control angles and cells in x and y
    	for(int m=0; m<(N_theta_x_2); m++) {
        	for(int n=0; n<(N_phi_x_4); n++) {
            	//get the correct angle
	            int angle = n + (m*4*N_phi);

    	        for(int i=1; i<cells_x-1; i++) {
        	        for(int j=1; j<cells_y-1; j++) {

            	        //IF Z=0 is a specular boundary:
                	    if(spec_z_0 == 1) {

                    	    //if the k component of S is going into the boundary
                        	if(propvec[angle].S_k_ < 0 ) {
                            	//the 'f' of the boundary for this angle equals
                                	//the one just out of the boundary
	                            cell[i][j][0].f_angle[angle] =
                                                cell[i][j][1].f_angle[angle];
    	                    }
	                        //if the k component of S is not going into the
    	                    //boundary                        
							else if(propvec[angle].S_k_ > 0) {
	                            //find the corresponding angle for specular
    	                            //reflection

        	                    //get the angle in the opposite 'k' direction
            	                int m_ref;
                	            //
	                            if(m<N_theta) { //this should be the case
    	                            m_ref = 2*N_theta - m -1; // - 1;
        	                    }
            	                else { //m>=N_theta
                	                m_ref = 2*N_theta - m -1; // - 1;
                    	        }

                        	    //put the reflected angle in the correct form
	                            angle_spec = n + (m_ref*4*N_phi);
    	                        //f(i, j, bottom boundary, this angle) =
        	                        //f(i,j,cell right above this one,ref_angle)
            	                cell[i][j][0].f_angle[angle] =
                	                        cell[i][j][1].f_angle[angle_spec];
                        	}
	                    }
    	                //IF Z=cells_z is a specular boundary:
        	            if (spec_z_end == 1) {
	
    	                    //if the k component of S is going into the boundary
        	                if(propvec[angle].S_k_ > 0 ) {
            	                //the 'f' of the boundary for this angle equals
                	                //the one just out of the boundary
	                            cell[i][j][cells_z-1].f_angle[angle] =
                                        cell[i][j][cells_z-2].f_angle[angle];

	                        }
    	                    //if the k component of S is not going into the
        	                    //boundary
            	            else if(propvec[angle].S_k_ < 0) {
	
    	                        //find the corresponding angle for specular
        	                        //reflection
                        	    //get the angle in the opposite 'k' direction
                            	int m_ref;
	                            if(m<N_theta) { //should not be the case
    	                            m_ref = 2*N_theta - m -1; // - 1;
        	                    }
            	                else { //m>=N_theta
                    	            m_ref = 2*N_theta - m -1;
                        	    }

	                            //put the reflected angle in the correct form
    	                        //angle_spec = angle_spec + (m_ref*4*N_phi);
        	                    angle_spec = n + (m_ref*4*N_phi);
            	                //f(i, j, top boundary, this angle) =
                	                //f(i, j, cell below this one, ref_angle)
                    	        cell[i][j][cells_z-1].f_angle[angle] =
                                    cell[i][j][cells_z-2].f_angle[angle_spec];
							}
                        }
                    }
                }
            }
        }
    }

}

////-------------------End of Specular boundaries code----------------////


//*==========================================================================*/
//* update the e0 and temp for the specular and diffuse boundaries
void BTE_structure::calc_spec_diff_e0_t() {

    //if a boundary in x is specular or diffuse
    if((spec_x_0==1)||(spec_x_end==1)||(diff_x_0==1)||(diff_x_end==1)) {
        //go over all cells in y and z
        for(int j=1; j<cells_y-1; j++) {
            for(int k=1; k<cells_z-1; k++) {
                if((spec_x_0==1)||(diff_x_0==1)) {
                    cell[0][j][k].T_ = cell[1][j][k].T_;
                    cell[0][j][k].e0_ =cell[1][j][k].e0_;
                }
                if((spec_x_end==1)||(diff_x_end==1)) {
                    cell[cells_x-1][j][k].T_ = cell[cells_x-2][j][k].T_;
                    cell[cells_x-1][j][k].e0_ = cell[cells_x-2][j][k].e0_;
                }
            }
        }
    }

    //if a boundary in y is specular or diffuse
    if((spec_y_0==1)||(spec_y_end==1)||(diff_y_0==1)||(diff_y_end==1)) {
        //go over all cells in z and x
        for(int i=1; i<cells_x-1; i++) {
            for(int k=1; k<cells_z-1; k++) {
                if((spec_y_0==1)||(diff_y_0==1)) {
                    cell[i][0][k].T_ = cell[i][1][k].T_;
                    cell[i][0][k].e0_ =cell[i][1][k].e0_;
                }
                if((spec_y_end==1)||(diff_y_end==1)) {
                    cell[i][cells_y-1][k].T_ = cell[i][cells_y-2][k].T_;
                    cell[i][cells_y-1][k].e0_ = cell[i][cells_y-2][k].e0_;
                }
            }
        }
    }

    //if a boundary in z is specular or diffuse
    if((spec_z_0==1)||(spec_z_end==1)||(diff_z_0==1)||(diff_z_end==1)) {
        //go over all cells in y and x
        for(int i=1; i<cells_x-1; i++) {
            for(int j=1; j<cells_y-1; j++) {
                if((spec_z_0==1)||(diff_z_0==1)) {
                    cell[i][j][0].T_ = cell[i][j][1].T_;
                    cell[i][j][0].e0_ =cell[i][j][1].e0_;
                }
                if((spec_z_end==1)||(diff_z_end==1)) {
                    cell[i][j][cells_z-1].T_ = cell[i][j][cells_z-2].T_;
                    cell[i][j][cells_z-1].e0_ = cell[i][j][cells_z-2].e0_;
                }
            }
        }
    }


}


////-------------------Start of Diffuse boundaries code----------------////

//*==========================================================================*/
//* updates the energies at diffuse boundaries
void BTE_structure::diffuse() {

	double e_in1; //used for west,south,and bottom boundaries
	double e_in2; //used for east,north,and top boundaries


	//if the structure has diffuse boundaries in 'y' at the 0 boundary
    if(diff_y_0 == 1 ) {
		//go over the x and z cells
		for(int i=1; i<cells_x-1; i++) {
	        for(int k=1; k<cells_z-1; k++) {
				//reset the energy values to zero
				e_in1 = 0;
				
				//go over all control angles
				for(int angle=0; angle<Nmax_angle; angle++) {
					//if the j component of S is going into the boundary
                    if(propvec[angle].S_j_ < 0 ) {
							e_in1 = e_in1 - (cell[i][1][k].f_angle[angle]*
														propvec[angle].S_j_);
					}
				}
				
				//rescale the energy values
				e_in1 = e_in1/PI;

				//go over all the angles again
                for(int angle=0; angle<Nmax_angle; angle++) {
					if(propvec[angle].S_j_ >= 0 ) {
                            cell[i][0][k].f_angle[angle] = e_in1;
					}
				}
            }
        }
    }

	//if the structure has diffuse boundaries in 'y' at the end boundary
    if(diff_y_end == 1) {
        //go over the x and z cells
        for(int i=1; i<cells_x-1; i++) {
            for(int k=1; k<cells_z-1; k++) {
                //reset the energy values to zero
                e_in2 = 0;

                //go over all control angles
                for(int angle=0; angle<Nmax_angle; angle++) {
                    //if the j component of S is going into the boundary
                    if(propvec[angle].S_j_ >= 0 ) {
                            e_in2 =e_in2+(cell[i][cells_y-2][k].f_angle[angle]*
                                                        propvec[angle].S_j_);
                    }
                }

                //rescale the energy values
                e_in2 = e_in2/PI;

                //go over all the angles again
                for(int angle=0; angle<Nmax_angle; angle++) {
                    if(propvec[angle].S_j_ < 0 ) {
                            cell[i][cells_y-1][k].f_angle[angle] = e_in2;
                    }
                }
            }
        }
    }


    //if the structure has diffuse boundaries in 'x' at the 0 boundary
    if(diff_x_0 == 1 ) {
        //go over the y and z cells
        for(int j=1; j<cells_y-1; j++) {
            for(int k=1; k<cells_z-1; k++) {
                //reset the energy values to zero
                e_in1 = 0;

                //go over all control angles
                for(int angle=0; angle<Nmax_angle; angle++) {
                    //if the i component of S is going into the boundary
                    if(propvec[angle].S_i_ < 0 ) {
                            e_in1 = e_in1 - (cell[1][j][k].f_angle[angle]*
                                                        propvec[angle].S_i_);
                    }
                }

                //rescale the energy values
                e_in1 = e_in1/PI;

                //go over all the angles again
                for(int angle=0; angle<Nmax_angle; angle++) {
                    if(propvec[angle].S_i_ >= 0 ) {
                            cell[0][j][k].f_angle[angle] = e_in1;
                    }
                }
            }
        }
    }

    //if the structure has diffuse boundaries in 'x' at the end boundary
    if(diff_x_end == 1) {
        //go over the y and z cells
        for(int j=1; j<cells_y-1; j++) {
            for(int k=1; k<cells_z-1; k++) {
                //reset the energy values to zero
                e_in2 = 0;

                //go over all control angles
                for(int angle=0; angle<Nmax_angle; angle++) {
					//if the i component of S is going into the boundary
                    if(propvec[angle].S_i_ >= 0 ) {
                            e_in2 =e_in2+(cell[cells_x-2][j][k].f_angle[angle]*
                                                        propvec[angle].S_i_);
                    }
                }

                //rescale the energy values
                e_in2 = e_in2/PI;

                //go over all the angles again
                for(int angle=0; angle<Nmax_angle; angle++) {
                    if(propvec[angle].S_i_ < 0 ) {
                            cell[cells_x-1][j][k].f_angle[angle] = e_in2;
                    }
                }
            }
        }
    }



    //if the structure has diffuse boundaries in 'z' at the 0 boundary
    if(diff_z_0 == 1 ) {
        //go over the x and y cells
        for(int i=1; i<cells_x-1; i++) {
            for(int j=1; j<cells_y-1; j++) {
                //reset the energy values to zero
                e_in1 = 0;

                //go over all control angles
                for(int angle=0; angle<Nmax_angle; angle++) {
                    //if the k component of S is going into the boundary
                    if(propvec[angle].S_k_ < 0 ) {
                            e_in1 = e_in1 - (cell[i][j][1].f_angle[angle]*
                                                        propvec[angle].S_k_);
                    }
                }

                //rescale the energy values
                e_in1 = e_in1/PI;

                //go over all the angles again
                for(int angle=0; angle<Nmax_angle; angle++) {
                    if(propvec[angle].S_k_ >= 0 ) {
                            cell[i][j][0].f_angle[angle] = e_in1;
                    }
                }
            }
        }
    }

    //if the structure has diffuse boundaries in 'z' at the end boundary
    if(diff_z_end == 1) {
        //go over the x and y cells
        for(int i=1; i<cells_x-1; i++) {
            for(int j=1; j<cells_y-1; j++) {
                //reset the energy values to zero
                e_in2 = 0;

                //go over all control angles
                for(int angle=0; angle<Nmax_angle; angle++) {
                    //if the k component of S is going into the boundary
                    if(propvec[angle].S_k_ >= 0 ) {
                            e_in2 =e_in2+(cell[i][j][cells_z-2].f_angle[angle]*
                                                        propvec[angle].S_k_);
                    }
                }

                //rescale the energy values
                e_in2 = e_in2/PI;

                //go over all the angles again
                for(int angle=0; angle<Nmax_angle; angle++) {
                    if(propvec[angle].S_k_ < 0 ) {
                            cell[i][j][cells_z-1].f_angle[angle] = e_in2;
                    }
                }
            }
        }
    }

}



////-------------------End of Diffuse boundaries code----------------////


//*==========================================================================*/
//* set the conductances of the boundary cells to zero if they are insulating
//* boundaries
void Fourier_structure::f_ins() {

    //if there are any insulating boundaries for this Fourier block in the x
    //direction
    if((ins_x_0==1)||(ins_x_end==1)) {
        //go through all cells at that boundary and set their conductance
        //to zero
        //go over the y and z cells
        for(int j=1; j<f_cells_y-1; j++) {
            for(int k=1; k<f_cells_z-1; k++) {
                //if the insulating boundary is on the x=0 boundary
                if(ins_x_0==1) {
                    //the conductance between the boundary and the next cell
                    //over is set to 0
                    f_cell[0][j][k].AEast_ = 0;
                }

                //if the insulating boundary is on the x=length_x boundary
                if(ins_x_end==1) {
                    //the conductance between the boundary and the next cell
                    //over is set to 0
                    f_cell[f_cells_x-2][j][k].AEast_ = 0;
                    //-2 because AEast is the conductance between the boundary
                    //and the last non-boundary cell
                }
            }
        }
    }
    //if there are any insulating boundaries for this Fourier block in the y
    //direction
    if((ins_y_0==1)||(ins_y_end==1)) {
        //go through all cells at that boundary and set their conductance
        //to zero
        //go over the x and z cells
        for(int i=1; i<f_cells_x-1; i++) {
            for(int k=1; k<f_cells_z-1; k++) {
                //if the insulating boundary is on the y=0 boundary
                if(ins_y_0==1) {
                    //the conductance between the boundary and the next cell
                    //over is set to 0
                    f_cell[i][0][k].ANorth_ = 0;
                }

                //if the insulating boundary is on the y=length_y boundary
                if(ins_y_end==1) {
                    //the conductance between the boundary and the next cell
                    //over is set to 0
                    f_cell[i][f_cells_y-2][k].ANorth_ = 0;
                }
            }
        }
    }
    //if there are any insulating boundaries for this Fourier block in the z
    //direction
    if((ins_z_0==1)||(ins_z_end==1)) {
        //go through all cells at that boundary and set their conductance
        //to zero
        //go over the x and y cells
        for(int i=1; i<f_cells_x-1; i++) {
            for(int j=1; j<f_cells_y-1; j++) {
                //if the insulating boundary is on the z=0 boundary
                if(ins_z_0==1) {
                    //the conductance between the boundary and the next cell
                    //over is set to 0
                    f_cell[i][j][0].ATop_ = 0;
                }

                //if the insulating boundary is on the z=length_z boundary
                if(ins_z_end==1) {
                    //the conductance between the boundary and the next cell
                    //over is set to 0
                    f_cell[i][j][f_cells_z-2].ATop_ = 0;
                }
            }
        }
    }

}



////-----End of Insulating boundaries code for Fourier cells-------////


////----------------Semi-reflecting boundaries-------------------////


//*==========================================================================*/
//* update the boundary conditions for the Fourier region for semi-reflecting
//* boundaries
void Fourier_structure::update_Fourier_semireflect(BTE_structure & bte_system,
												Block_coords & bte_coords) {

	//check if this structure has any semi reflecting boundaries
	//if yes update those q values (check which direction and which cells)

	int cells_y = bte_system.cells_y;
    int cells_z = bte_system.cells_z;
	int cells_x = bte_system.cells_x;

/*
    for(int i=1;i<f_cells_x-1;i++)
    {
        for(int j=1;j<f_cells_y-1;j++)
        {
            for(int k=1;k<f_cells_z-1;k++)
            {
                f_cell[i][j][k].Bprime = 0;
            }
        }
    }
*/
        
    //need to get the boundary cells to know which ones they are together with

	//go over all the possible semireflecting boundaries
	if(bte_system.oxide_x_0) { //west side of BTE is reflecting, 
		//so n = S_i is negative
		//calculate q going into the fourier region
		//q=G-J

		//need to go over all the cells at the boundary
		for(int j=1; j<cells_y-1; j++) {
			for(int k=1; k<cells_z-1; k++) {
				//using cells at the BTE edge (one away from the boundary)
				//cell[1][j][k]

				//G = SUM(vg*e''*s.n) over all angle extents
				double G = 0;
				double E_term=0;
				int Nmax_angle = bte_system.Nmax_angle;
				for(int l=0; l<Nmax_angle; l++) {
					//if the angle is in the right direction
					if(bte_system.propvec[l].S_i_<0) {

						//get the G contribution from this angle	
						G = G + bte_system.cell[1][j][k].f_angle[l]*
						(-bte_system.propvec[l].S_i_);
					}
					else {
						E_term = E_term -bte_system.propvec[l].S_i_;
					}
					
				}

				//get the G and E_term times the constants
				G = G*bte_system.cell[1][j][k].material->vg_;
				//E_wall is the e0 of the boundary bte cell
				E_term = E_term*bte_system.cell[1][j][k].material->vg_*
											bte_system.cell[0][j][k].e0_;
				double J = G*(1-bte_system.trans_factor[0]) - 
									E_term*(bte_system.trans_factor[0]);
				
				double Q = G - J;
				//get the Q (heat source) in Watts, based on the area of the 
				//cell it is comming through
				//The area of the cell may need to be changed if a non-uniform
				//grid is used
				Q = Q*bte_system.del_y*bte_system.del_z;
			
				//get the fourier cell at the west boundary (h=1) of the BTE 
				//cell, and set its new power number
				bte_system.cell[0][j][k].cell_boundaries[1]->Qprime =Q;

                //Teq = Tfcell + Q/K
                double Teq =
                    bte_system.cell[0][j][k].cell_boundaries[1]->T_ +
                Q/(bte_system.cell[0][j][k].cell_boundaries[1]->AEast_);

                bte_system.cell[0][j][k].cell_boundaries[1]->Bprime =
bte_system.cell[0][j][k].cell_boundaries[1]->Bprime +
                         bte_system.cell[0][j][k].cell_boundaries[1]->AEast_*
                         Teq;
                //set the temperature to the right cell
                f_cell[bte_coords.x][bte_coords.y+j-1]
                                                [bte_coords.z+k-1].T_ = 0;
//Teq;

			}
			
		}
	}
	
	if(bte_system.oxide_x_end) { //east side of BTE is reflecting, 
		//so n = S_i is positive
        //calculate q going into the fourier region
        //q=G-J

        //need to go over all the cells at the boundary
        for(int j=1; j<cells_y-1; j++) {
            for(int k=1; k<cells_z-1; k++) {
                //using cells at the BTE edge (one away from the boundary)
                //cell[1][j][k]

                //G = SUM(vg*e''*s.n) over all angle extents
                double G = 0;
                double E_term=0;
                int Nmax_angle = bte_system.Nmax_angle;
                for(int l=0; l<Nmax_angle; l++) {
                    //if the angle is in the right direction
                    if(bte_system.propvec[l].S_i_>0) {

                        //get the G contribution from this angle
                        G = G + bte_system.cell[cells_x-2][j][k].f_angle[l]*
                        bte_system.propvec[l].S_i_;
					}
					else {
                        E_term = E_term + bte_system.propvec[l].S_i_;

                    }

                }

                //get the G and E_term times the constants
                G = G*bte_system.cell[cells_x-2][j][k].material->vg_;
                //E_wall is the e0 of the boundary bte cell
                E_term = E_term*bte_system.cell[cells_x-2][j][k].material->vg_*
                                        bte_system.cell[cells_x-1][j][k].e0_;
                double J = G*(1-bte_system.trans_factor[1]) -
                                    E_term*(bte_system.trans_factor[1]);

                double Q = G - J;
                //get the Q (heat source) in Watts, based on the area of the
                //cell it is comming through
                //The area of the cell may need to be changed if a non-uniform
                //grid is used
                Q = Q*bte_system.del_y*bte_system.del_z;

                //get the fourier cell at the west boundary (h=1) of the BTE
                //cell, and set its new power number

				bte_system.cell[cells_x-1][j][k].cell_boundaries[0]->Qprime =Q;

                //Teq = Tfcell + Q/K
                double Teq =
                    bte_system.cell[cells_x-1][j][k].cell_boundaries[0]->T_ +
                Q/(f_cell[bte_coords.x+cells_x-3][bte_coords.y+j-1]
                                                [bte_coords.z+k-1].AEast_);

                bte_system.cell[cells_x-1][j][k].cell_boundaries[0]->Bprime =
bte_system.cell[cells_x-1][j][k].cell_boundaries[0]->Bprime +
                         f_cell[bte_coords.x+cells_x-3][bte_coords.y+j-1]
                                                [bte_coords.z+k-1].AEast_*
                         Teq;
                //set the temperature to the right cell
                f_cell[bte_coords.x+cells_x-3][bte_coords.y+j-1]
                                                [bte_coords.z+k-1].T_ = 0;
//Teq;

            }

        }
    }


    if(bte_system.oxide_y_0) { //south side of BTE is reflecting, 
		//so n = S_j is negative
        //calculate q going into the fourier region
        //q=G-J

        //need to go over all the cells at the boundary
        for(int i=1; i<cells_x-1; i++) {
            for(int k=1; k<cells_z-1; k++) {
                //using cells at the BTE edge (one away from the boundary)

                //G = SUM(vg*e''*s.n) over all angle extents
                double G = 0;
                double E_term=0;
                int Nmax_angle = bte_system.Nmax_angle;
                for(int l=0; l<Nmax_angle; l++) {
                    //if the angle is in the right direction
                    if(bte_system.propvec[l].S_j_<0) {

                        //get the G contribution from this angle
                        G = G + bte_system.cell[i][1][k].f_angle[l]*
                        (-bte_system.propvec[l].S_j_ );
					}
					else {
                        E_term = E_term -bte_system.propvec[l].S_j_;

                    }

                }

                //get the G and E_term times the constants
                G = G*bte_system.cell[i][1][k].material->vg_;
                //E_wall is the e0 of the boundary bte cell
                E_term = E_term*bte_system.cell[i][1][k].material->vg_*
                                        bte_system.cell[i][0][k].e0_;
                double J = G*(1-bte_system.trans_factor[2]) -
                                    E_term*(bte_system.trans_factor[2]);

                double Q = G - J;
                //get the Q (heat source) in Watts, based on the area of the
                //cell it is comming through
                //The area of the cell may need to be changed if a non-uniform
                //grid is used
                Q = Q*bte_system.del_x*bte_system.del_z;

                //get the fourier cell at the west boundary (h=1) of the BTE
                //cell, and set its new power number
				
				bte_system.cell[i][0][k].cell_boundaries[3]->Qprime =Q;

                //Teq = Tfcell + Q/K
                double Teq =
                    bte_system.cell[i][0][k].cell_boundaries[3]->T_ +
                Q/(bte_system.cell[i][0][k].cell_boundaries[3]->ANorth_);

                bte_system.cell[i][0][k].cell_boundaries[3]->Bprime =
bte_system.cell[i][0][k].cell_boundaries[3]->Bprime + 
                         bte_system.cell[i][0][k].cell_boundaries[3]->ANorth_*
                         Teq;

                //set the temperature to the right cell
                f_cell[bte_coords.x+i-1][bte_coords.y]
                                                [bte_coords.z+k-1].T_ = 0;
//Teq;
            }

        }
    }
	if(bte_system.oxide_y_end) { //north side of BTE is reflecting, 
		//so n = S_j is positive
        //calculate q going into the fourier region
        //q=G-J

        //need to go over all the cells at the boundary
        for(int i=1; i<cells_x-1; i++) {
            for(int k=1; k<cells_z-1; k++) {
                //using cells at the BTE edge (one away from the boundary)

                //G = SUM(vg*e''*s.n) over all angle extents
                double G = 0;
                double E_term=0;
                int Nmax_angle = bte_system.Nmax_angle;
                for(int l=0; l<Nmax_angle; l++) {
                    //if the angle is in the right direction
                    if(bte_system.propvec[l].S_j_>0) {

                        //get the G contribution from this angle
                        G = G + bte_system.cell[i][cells_y-2][k].f_angle[l]*
                        bte_system.propvec[l].S_j_;
					}
					else {
                        E_term = E_term + bte_system.propvec[l].S_j_;

                    }

                }

                //get the G and E_term times the constants
                G = G*bte_system.cell[i][cells_y-2][k].material->vg_;
                //E_wall is the e0 of the boundary bte cell
                E_term = E_term*bte_system.cell[i][cells_y-2][k].material->vg_*
                                        bte_system.cell[i][cells_y-1][k].e0_;
                double J = G*(1-bte_system.trans_factor[3]) -
                                    E_term*(bte_system.trans_factor[3]);

                double Q = G - J;
                //get the Q (heat source) in Watts, based on the area of the
                //cell it is comming through
                //The area of the cell may need to be changed if a non-uniform
                //grid is used
                Q = Q*bte_system.del_x*bte_system.del_z;

                //get the fourier cell at the west boundary (h=1) of the BTE
                //cell, and set its new power number
				bte_system.cell[i][cells_y-1][k].cell_boundaries[2]->Qprime =Q;

                //Teq = Tfcell + Q/K
                double Teq =
                    bte_system.cell[i][cells_y-1][k].cell_boundaries[2]->T_ +
                Q/(f_cell[bte_coords.x+i-1][bte_coords.y+cells_y-3]
                                                [bte_coords.z+k-1].ANorth_);
                bte_system.cell[i][cells_y-1][k].cell_boundaries[2]->Bprime =
bte_system.cell[i][cells_y-1][k].cell_boundaries[2]->Bprime + 
                        f_cell[bte_coords.x+i-1][bte_coords.y+cells_y-3]
                                                [bte_coords.z+k-1].ANorth_*
                        Teq;

                //set the temperature to the right cell
                f_cell[bte_coords.x+i-1][bte_coords.y+cells_y-3]
                                                [bte_coords.z+k-1].T_ = 0;
//Teq;
            }

        }
    }

	if(bte_system.oxide_z_0) { //bottom side of BTE is reflecting, 
		//so n = S_k is negative
        //calculate q going into the fourier region
        //q=G-J

        //need to go over all the cells at the boundary
        for(int i=1; i<cells_x-1; i++) {
            for(int j=1; j<cells_y-1; j++) {
                //using cells at the BTE edge (one away from the boundary)

                //G = SUM(vg*e''*s.n) over all angle extents
                double G = 0;
                double E_term=0;
                int Nmax_angle = bte_system.Nmax_angle;
                for(int l=0; l<Nmax_angle; l++) {
                    //if the angle is in the right direction
                    if(bte_system.propvec[l].S_k_<0) {

                        //get the G contribution from this angle
                        G = G + bte_system.cell[i][j][1].f_angle[l]*
                        (-bte_system.propvec[l].S_k_);
					}
					else if(bte_system.propvec[l].S_k_>0) {
                        E_term = E_term + bte_system.propvec[l].S_k_;

                    }

                }

                //get the G and E_term times the constants
				G = G*bte_system.cell[i][j][1].material->vg_;

                //E_wall is the e0 of the boundary bte cell
			    E_term = E_term*bte_system.cell[i][j][1].material->vg_
						*bte_system.cell[i][j][0].e0_;

                double J = G*(1-bte_system.trans_factor[4]) +
                                    E_term*(bte_system.trans_factor[4]);

                double Q = G - J;

                //get the Q (heat source) in Watts, based on the area of the
                //cell it is comming through
                //The area of the cell may need to be changed if a non-uniform
                //grid is used
                Q = Q*bte_system.del_x*bte_system.del_y;

                //get the fourier cell at the west boundary (h=1) of the BTE
                //cell, and set its new power number

				bte_system.cell[i][j][0].cell_boundaries[5]->Qprime = Q;

				//instead of using Q as a boundary, set the temperature of the
				//boundaries of the Fourier cells to the equivalent temperature
				//using the heat comming from the boundary
				//where Teq = Tfcell + Q/K
				double Teq = 
					bte_system.cell[i][j][0].cell_boundaries[5]->T_ +
					Q/(bte_system.cell[i][j][0].cell_boundaries[5]->ATop_);

                bte_system.cell[i][j][0].cell_boundaries[5]->Bprime =
bte_system.cell[i][j][0].cell_boundaries[5]->Bprime +
                        bte_system.cell[i][j][0].cell_boundaries[5]->ATop_*
                        Teq;
				//set the temperature to the right cell				
				f_cell[bte_coords.x+i-1][bte_coords.y+j-1][bte_coords.z].T_ = 0;
//Teq;
            }

        }
    }
	if(bte_system.oxide_z_end) { //top side of BTE is reflecting, 
		//so n = S_k is positive
        //calculate q going into the fourier region
        //q=G-J

        //need to go over all the cells at the boundary
        for(int i=1; i<cells_x-1; i++) {
            for(int j=1; j<cells_y-1; j++) {
                //using cells at the BTE edge (one away from the boundary)

                //G = SUM(vg*e''*s.n) over all angle extents
                double G = 0;
                double E_term=0;
                int Nmax_angle = bte_system.Nmax_angle;
                for(int l=0; l<Nmax_angle; l++) {
                    //if the angle is in the right direction
                    if(bte_system.propvec[l].S_k_>0) {

                        //get the G contribution from this angle
                        G = G + bte_system.cell[i][j][cells_z-2].f_angle[l]*
                        bte_system.propvec[l].S_k_;
					}
					else {
                        E_term = E_term + bte_system.propvec[l].S_k_;
                    }

                }

                //get the G and E_term times the constants
                G = G*bte_system.cell[i][j][cells_z-2].material->vg_;
                //E_wall is the e0 of the boundary bte cell
                E_term = E_term*bte_system.cell[i][j][cells_z-2].material->vg_
                                        *bte_system.cell[i][j][cells_z-1].e0_;

                double J = G*(1-bte_system.trans_factor[5]) -
                                    E_term*(bte_system.trans_factor[5]);

                double Q = G - J;
                //get the Q (heat source) in Watts, based on the area of the
                //cell it is comming through
                //The area of the cell may need to be changed if a non-uniform
                //grid is used
                Q = Q*bte_system.del_x*bte_system.del_y;

                //get the fourier cell at the west boundary (h=1) of the BTE
                //cell, and set its new power number
				bte_system.cell[i][j][cells_z-1].cell_boundaries[4]->Qprime =Q;

                //Teq = Tfcell + Q/K
                double Teq =
                    bte_system.cell[i][j][cells_z-1].cell_boundaries[4]->T_ +
                Q/(f_cell[bte_coords.x+i-1][bte_coords.y+j-1]
                                      [bte_coords.z+cells_z-3].ATop_);

                bte_system.cell[i][j][cells_z-1].cell_boundaries[4]->Bprime =
bte_system.cell[i][j][cells_z-1].cell_boundaries[4]->Bprime +
                            f_cell[bte_coords.x+i-1][bte_coords.y+j-1]
                                      [bte_coords.z+cells_z-3].ATop_*
                        Teq;
				//bte_system.cell[i][j][cells_z-1].cell_boundaries[4]->ATop_*2);
				//changed it from that, because that equation is not the 
				//general case (the ATop used should be the one below this, 
				//which is what is currently being used)

                //set the temperature to the right cell
                f_cell[bte_coords.x+i-1][bte_coords.y+j-1]
									[bte_coords.z+cells_z-3].T_ = 0;
//Teq;

            }

        }
    }

}

//*==========================================================================*/
void Device_structure::update_bte_semireflect() {

    int size = bte_system.size();

    for(int h=0;h<size;h++)
    {
        bte_system[h].update_bte_semireflect();
    }
}

//*==========================================================================*/
//* For updating the values in BTE (before BTE starts)
void BTE_structure::update_bte_semireflect() {

//for energy incomming into BTE region
//ONLY doing diffuse
//e'' = ((1/pi)*(1-ai)*(SUM(e''*s.n) over all angles in n direction) + ai*ewall
    double e_temp;

    //if there is a silicon dioxide layer to the west
    if(oxide_x_0==1)
    {
        //loop over all BTE cells touching that oxide layer
        for(int j=1;j<cells_y-1;j++)
        {
            for(int k=1;k<cells_z-1;k++)
            {    
                e_temp=0;
                //calculate SUM(e''*s.n)
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_i_<0)
                    {
                        e_temp=e_temp+cell[1][j][k].f_angle[angle]*
                               (-propvec[angle].S_i_);
                    }
                }
                //scale by (1/pi)*(1-ai)
                e_temp=e_temp/(PI)*(1-trans_factor[0]);
                //add the ai*ewall term
                e_temp=e_temp+trans_factor[0]*cell[0][j][k].e0_;
                //set all f_angles going out of the interface to this e'' value
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_i_>0)
                    {
                        cell[0][j][k].f_angle[angle]=e_temp;
                    }
                }
            }
        }
    }
    //if there is a silicon dioxide layer to the east
    if(oxide_x_end==1)
    {
        //loop over all BTE cells touching that oxide layer
        for(int j=1;j<cells_y-1;j++)
        {
            for(int k=1;k<cells_z-1;k++)
            {    
                e_temp=0;
                //calculate SUM(e''*s.n)
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_i_>0)
                    {
                        e_temp=e_temp+cell[cells_x-2][j][k].f_angle[angle]*
                               (propvec[angle].S_i_);
                    }
                }
                //scale by (1/pi)*(1-ai)
                e_temp=e_temp/(PI)*(1-trans_factor[1]);
                //add the ai*ewall term
                e_temp=e_temp+trans_factor[1]*cell[cells_x-1][j][k].e0_;
                //set all f_angles going out of the interface to this e'' value
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_i_<0)
                    {
                        cell[cells_x-1][j][k].f_angle[angle]=e_temp;
                    }
                }
            }
        }
    }
    //if there is a silicon dioxide layer to the south
    if(oxide_y_0==1)
    {
        //loop over all BTE cells touching that oxide layer
        for(int i=1;i<cells_x-1;i++)
        {
            for(int k=1;k<cells_z-1;k++)
            {    
                e_temp=0;
                //calculate SUM(e''*s.n)
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_j_<0)
                    {
                        e_temp=e_temp+cell[i][1][k].f_angle[angle]*
                               (-propvec[angle].S_j_);
                    }
                }
                //scale by (1/pi)*(1-ai)
                e_temp=e_temp/(PI)*(1-trans_factor[2]);
                //add the ai*ewall term
                e_temp=e_temp+trans_factor[2]*cell[i][0][k].e0_;
                //set all f_angles going out of the interface to this e'' value
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_j_>0)
                    {
                        cell[i][0][k].f_angle[angle]=e_temp;
                    }
                }
            }
        }
    }
    //if there is a silicon dioxide layer to the north
    if(oxide_y_end==1)
    {
        //loop over all BTE cells touching that oxide layer
        for(int i=1;i<cells_x-1;i++)
        {
            for(int k=1;k<cells_z-1;k++)
            {    
                e_temp=0;
                //calculate SUM(e''*s.n)
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_j_>0)
                    {
                        e_temp=e_temp+cell[i][cells_y-2][k].f_angle[angle]*
                               (propvec[angle].S_j_);
                    }
                }
                //scale by (1/pi)*(1-ai)
                e_temp=e_temp/(PI)*(1-trans_factor[3]);
                //add the ai*ewall term
                e_temp=e_temp+trans_factor[3]*cell[i][cells_y-1][k].e0_;
                //set all f_angles going out of the interface to this e'' value
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_j_<0)
                    {
                        cell[i][cells_y-1][k].f_angle[angle]=e_temp;
                    }
                }
            }
        }
    }
                
    //if there is a silicon dioxide layer at the bottom
    if(oxide_z_0==1)
    {
        //loop over all BTE cells touching that oxide layer
        for(int j=1;j<cells_y-1;j++)
        {
            for(int i=1;i<cells_x-1;i++)
            {    
                e_temp=0;
                //calculate SUM(e''*s.n)
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_k_<0)
                    {
                        e_temp=e_temp+cell[i][j][1].f_angle[angle]*
                               (-propvec[angle].S_k_);
                    }
                }
                //scale by (1/pi)*(1-ai)
                e_temp=e_temp/(PI)*(1-trans_factor[4]);
                //add the ai*ewall term
                e_temp=e_temp+trans_factor[4]*cell[i][j][0].e0_;
                //set all f_angles going out of the interface to this e'' value
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_k_>0)
                    {
                        cell[i][j][0].f_angle[angle]=e_temp;
                    }
                }
            }
        }
    }
                
    //if there is a silicon dioxide layer on the top
    if(oxide_z_end==1)
    {
        //loop over all BTE cells touching that oxide layer
        for(int j=1;j<cells_y-1;j++)
        {
            for(int i=1;i<cells_x-1;i++)
            {    
                e_temp=0;
                //calculate SUM(e''*s.n)
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_k_>0)
                    {
                        e_temp=e_temp+cell[i][j][cells_z-2].f_angle[angle]*
                               (propvec[angle].S_k_);
                    }
                }
                //scale by (1/pi)*(1-ai)
                e_temp=e_temp/(PI)*(1-trans_factor[5]);
                //add the ai*ewall term
                e_temp=e_temp+trans_factor[5]*cell[i][j][cells_z-1].e0_;
                //set all f_angles going out of the interface to this e'' value
                for(int angle=0;angle<Nmax_angle;angle++)
                {
                    if(propvec[angle].S_k_<0)
                    {
                        cell[i][j][cells_z-1].f_angle[angle]=e_temp;
                    }
                }
            }
        }
    }
}


////-------------End of semi-reflecting boundaries---------------////



//*==========================================================================*/
//* check to make sure the reflecting boudaries don't hit walls
void Device_structure::check_reflecting_boundaries() {

	int size = bte_system.size();

	//check all oxide boundaries
	for(int h=0;h<size;h++) {
		
		//make sure they are not touching a system wall
		//if they are, make it so these are not reflecting walls
		
		if(bte_system[h].oxide_x_0) {
			//if the x coords touch the wall, then make it not reflecting
			if(bte_coords[h].x==1) {
				bte_system[h].oxide_x_0 = 0;
			}
		}
		if(bte_system[h].oxide_x_end) {
			//if the x coords touch the wall, then make it not reflecting
            if(bte_coords[h].x+bte_coords[h].cells_x==substrate.f_cells_x+1) {
                bte_system[h].oxide_x_end = 0;
            }
	
		}
		if(bte_system[h].oxide_y_0) {
            if(bte_coords[h].y==1) {
                bte_system[h].oxide_y_0 = 0;
            }

		}
		if(bte_system[h].oxide_y_end) {
            if(bte_coords[h].y+bte_coords[h].cells_y==substrate.f_cells_y+1) {
                bte_system[h].oxide_y_end = 0;
            }

		}
		if(bte_system[h].oxide_z_0) {
            if(bte_coords[h].z==1) {
                bte_system[h].oxide_z_0 = 0;
            }

		}
		if(bte_system[h].oxide_z_end) {
            if(bte_coords[h].z+bte_coords[h].cells_z==substrate.f_cells_z+1) {
                bte_system[h].oxide_z_end = 0;
            }		
		}
	}
}


