/****************************************************************************
**
** ISAC2: Integrated Space and Time Adaptive Chip-Package Thermal Analysis, 
**        version 2.2
**
** Copyright (C) 2018 University of Colorado at Boulder and University 
** of Michigan at Ann Arbor. All rights reserved.
**
** This file is part of ISAC2.
**
** ISAC2 is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, version 2 of the License.
**
** ISAC2 is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with ISAC2 in the file COPYING.  If not, see 
** <http://www.gnu.org/licenses/>.
**
** For feedback, comments, or questions, please contact Zyad Hassan 
** (zyad.hassan@colorado.edu) and Xi Chen (chexi@umich.edu)
**
** Please contact the authors if you are interested in making commercial
** use of ISAC2.  It may be possible to re-release under a different license.
****************************************************************************/

/****************************************************************************
** cell.cc: contains implementations for functions in the class "Cell"
****************************************************************************/

#include "header.h"

//*==========================================================================*/
//* Cell class constructor
Cell::Cell()
{
	//initialize all variables with zeros
	
	AP_=AWest_=AEast_=ASouth_=ANorth_=ABottom_=ATop_=0;
	B_=0;
	e0_=T_=0;
	q_vol=0;
	on_time = offset_time = 0;
	q_vol_on=0;
}

//*==========================================================================*/
//* calculate this cells e0 and T
void Cell::calc_cell_e0_t(vector< Propagation > &propvec) {

    //set this cell's e0_ to 0
    e0_ = 0;

    //get amount of angles
    int size = propvec.size();

    //go over all angular directions
    for(int l=0; l<size; l++) {

        //e0_ = e0_ + f(this cell, current angle)*extent(current angle)
        e0_ = e0_ + f_angle[l]*propvec[l].extent();
    }
    //set the temperature
	T_ = e0_/material->cv_ + material->t_ref_;

    //set e0 to correct units
    e0_ = e0_/(4*PI);
}

//*==========================================================================*/
//* reset the coefficients to 0
void Cell::reset_coeffs(void) {
     AP_ = AWest_ = AEast_ = ASouth_ = ANorth_ = ABottom_ = ATop_ = B_ = 0;
}

//*==========================================================================*/
//* set the guesses and conditions of the cell
void Cell::ini_cell_solve(BTE_structure & system, double temper) {

    //set initial temperatures for all cells
	T_ = temper;

    //set angular average of energy density of all cells
	e0_ = material->cv_*(T_ -  material->t_ref_)/(4*PI);

    //set the energy densities of the directions
    for(int i=0; i<system.Nmax_angle; i++) {
        f_angle[i] = e0_;
    }

}

//*==========================================================================*/
//* set the conditions of the cell if it has a Fourier boundary OR if the fourier
//* solver was run for the first time to get the BTE initial conditions
void Cell::fbound_cell_solve(int Nmax_angle) {

    //set initial temperatures for all cells
	//set angular average of energy density of all cells
	e0_ = material->cv_*(T_ -  material->t_ref_)/(4*PI);

    //set the energy densities of the directions
    for(int i=0; i<Nmax_angle; i++) {
//        f_angle[i] = e0_;
    }

}

/*
//set the guesses and conditions of the cell for Fourier cells
void Cell::ini_cell_solve_fourier(Structure & system, double temper) {

    //set initial temperatures for all cells
    T_ = temper;

}
*/


/*----------------Fourier cell functions----------------------*/

//*==========================================================================*/
//* Fourier Cell class constructor
Fourier_cell::Fourier_cell()
{
	//initialize all variables with zeros
	
	AP_=AEast_=ANorth_=ATop_=0;
	B_=0;
	T_=0;
	q_vol=0;
	on_time = offset_time = 0;
	q_vol_on=0;

//	Qprime=0;
	
}

//*==========================================================================*/
//* reset the coefficients to 0
void Fourier_cell::reset_coeffs(void) {
     AP_ = AEast_ = ANorth_ = ATop_ = B_ = 0;
}

//*==========================================================================*/
//* set the guesses and conditions of the cell for Fourier cells
void Fourier_cell::ini_cell_solve_fourier(double temper) {

    //set initial temperatures for all cells
    T_ = temper;

}

/*---------------End of Fourier cell functions----------------*/


